// Generated by CoffeeScript 1.12.7
var Converter, binaryDetect, escapeBasicHtml, htmlEntityAmp, htmlEntityGt, htmlEntityLt, htmlEntityQuot, jsonPointer, regExpAmp, regExpGt, regExpLt, regExpQuot, sanitizer, traverse,
  indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

binaryDetect = require('./binary-detect');

sanitizer = require('./html-sanitizer');

traverse = require('traverse');

jsonPointer = require('json-pointer');

regExpAmp = /&/g;

htmlEntityAmp = '&amp;';

regExpLt = /</g;

htmlEntityLt = '&lt;';

regExpGt = />/g;

htmlEntityGt = '&gt;';

regExpQuot = /"/g;

htmlEntityQuot = '&quot;';

escapeBasicHtml = function(val) {
  return ('' + val).replace(regExpAmp, htmlEntityAmp).replace(regExpLt, htmlEntityLt).replace(regExpGt, htmlEntityGt).replace(regExpQuot, htmlEntityQuot);
};

Converter = (function() {
  function Converter(arg) {
    this.dataReal = arg.dataReal, this.dataExpected = arg.dataExpected, this.fieldResult = arg.fieldResult, this.usePointers = arg.usePointers;
    this.usedErrors = [];
  }

  Converter.prototype.getHtml = function(options) {
    var html;
    if (options == null) {
      options = {};
    }
    this.wrapWith = options.wrapWith, this.startTag = options.startTag, this.jsonKeyStartTag = options.jsonKeyStartTag, this.jsonKeyEndTag = options.jsonKeyEndTag, this.missingStartTag = options.missingStartTag, this.addedStartTag = options.addedStartTag, this.changedStartTag = options.changedStartTag, this.endTag = options.endTag, this.comments = options.comments, this.commentStartTag = options.commentStartTag, this.commentEndTag = options.commentEndTag, this.identString = options.identString;
    this.outputs = [this.missingStartTag, this.startTag, this.addedStartTag, this.changedStartTag];
    html = this.getHtmlPrivate();
    if (this.wrapWith) {
      html = this.wrapWith.replace('##data', function() {
        return html;
      });
    }
    return html;
  };

  Converter.prototype.getHtmlPrivate = function() {
    throw new Error('getHtmlPrivate: not implemented. Must be implemented in subclass');
  };

  Converter.prototype.formatFragmentParts = function(arg) {
    var message, out, status;
    message = arg.message, status = arg.status;
    if (status == null) {
      status = 0;
    }
    out = ["" + (this.outputs[1 * (status || 0) + 1] || ''), [this.endTag]];
    if (this.comments && message) {
      out[1].push("" + this.commentStartTag + message + this.commentEndTag);
    }
    return out;
  };

  Converter.prototype.formatFragment = function(arg) {
    var fragment, message, omitSanitize, output, status;
    fragment = arg.fragment, message = arg.message, status = arg.status, omitSanitize = arg.omitSanitize;
    output = '';
    switch (status) {
      case -1:
        output += this.missingStartTag;
        break;
      case 0:
        output += this.startTag;
        break;
      case 1:
        output += this.addedStartTag;
        break;
      case 2:
        output += this.changedStartTag;
    }
    if (omitSanitize) {
      output += fragment;
    } else {
      output += this.sanitize(fragment);
    }
    output += this.endTag;
    if (this.comments && message) {
      output += ' ' + this.commentStartTag;
      output += message;
      output += this.commentEndTag;
    }
    return output;
  };

  Converter.prototype.buildDataRealPointers = function(arg) {
    var compiledPointers, compiledPointersKey, ref, transformKeysFn;
    compiledPointersKey = arg.compiledPointersKey, transformKeysFn = arg.transformKeysFn;
    if (!((ref = this.dataRealPointers) != null ? ref[compiledPointersKey] : void 0)) {
      compiledPointers = [];
      traverse(this.dataReal).forEach(function() {
        compiledPointers.push(jsonPointer.compile(this.path.map(transformKeysFn)));
      });
      if (this.dataRealPointers == null) {
        this.dataRealPointers = {};
      }
      return this.dataRealPointers[compiledPointersKey] = compiledPointers;
    }
  };

  Converter.prototype.getPointerTransformKeys = function(lowerCasedKeys) {
    var compiledPointersKey, transformKeysFn;
    transformKeysFn = this._lambda;
    compiledPointersKey = 'original';
    if (lowerCasedKeys) {
      transformKeysFn = this._lowerCaseIt;
      compiledPointersKey = 'lowercased';
    }
    return [transformKeysFn, compiledPointersKey];
  };

  Converter.prototype.getStateAndMessageFromResults = function(arg) {
    var compiledPointersKey, error, errorPointer, errorsCount, j, len, lowerCasedKeys, message, pathArray, pathArrayTransformed, ref, ref1, ref2, ref3, ref4, state, transformKeysFn;
    pathArray = arg.pathArray, lowerCasedKeys = arg.lowerCasedKeys;
    errorsCount = (ref = this.fieldResult) != null ? (ref1 = ref.errors) != null ? ref1.length : void 0 : void 0;
    state = 0;
    message = '';
    if (errorsCount) {
      ref2 = this.getPointerTransformKeys(lowerCasedKeys), transformKeysFn = ref2[0], compiledPointersKey = ref2[1];
      this.buildDataRealPointers({
        transformKeysFn: transformKeysFn,
        compiledPointersKey: compiledPointersKey
      });
      pathArrayTransformed = pathArray.map(transformKeysFn);
      ref3 = this.fieldResult.errors;
      for (j = 0, len = ref3.length; j < len; j++) {
        error = ref3[j];
        errorPointer = (ref4 = error.location) != null ? ref4.pointer : void 0;
        if (errorPointer != null) {
          if (this.areArraysIdentical(pathArrayTransformed, jsonPointer.parse(errorPointer))) {
            if (indexOf.call(this.dataRealPointers[compiledPointersKey], errorPointer) < 0) {
              if (message) {
                message = ' | ' + message;
              }
              message = error['message'] + message;
              state = -1;
            } else {
              if (!state) {
                state = 2;
              }
              if (message) {
                message += " | ";
              }
              message += error['message'];
            }
          }
        }
      }
    }
    return {
      pathArray: pathArray,
      'state': state,
      'message': message
    };
  };

  Converter.prototype.sanitize = function(data) {
    return binaryDetect(data, sanitizer, escapeBasicHtml);
  };

  Converter.prototype._lowerCaseIt = function(str) {
    return str.toLowerCase();
  };

  Converter.prototype._lambda = function(unknown) {
    return unknown;
  };

  Converter.prototype.areArraysIdentical = function(a1, a2) {
    var i;
    i = a1.length;
    if (i !== a2.length) {
      return false;
    }
    while (i--) {
      if (a1[i] !== a2[i]) {
        return false;
      }
    }
    return true;
  };

  return Converter;

})();

module.exports = Converter;
